/* stat.cc  Statistics Program for Motif */

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/SeparatoG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/Frame.h>
#include <Xm/LabelG.h>
#include <Xm/MessageB.h>
#include <Xm/List.h>
#include <Xbae/Matrix.h>
#include <X11/Xmu/Editres.h>
#include <X11/xpm.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include "icon.h"
#include "Toolbar.h"
#include "pixmaps/folder.xpm"
#include "pixmaps/arrdown.xpm"
#include "pixmaps/arrup.xpm"
#include "pixmaps/arrleft.xpm"
#include "pixmaps/arrright.xpm"
#include "pixmaps/delrow.xpm"
#include "pixmaps/delcol.xpm"

#define DISW 500           // display width;
#define DISH 300           // display hight;
#define COLWIDTH 10        // width of each column;
#define EMPTY ""           // empty cell;
#define VERSION "2.4"
#define OUTBUFLEN 100000
#define DATALEN 1000

/* Structure for menu system */
typedef struct {
  Widget f_menu;
  Widget n_data;
  Widget l_data;
  Widget s_data;
  Widget c_data;
  Widget quit;
} f_menu;
typedef struct {
  Widget e_menu;
  Widget addrow;
  Widget addrowb;
  Widget addrowa;
  Widget addrowe;
  Widget addcol;
  Widget addcolb;
  Widget addcola;
  Widget addcole;
  Widget cpcol;
  Widget cprow;
  Widget delrow;
  Widget delcol;
  Widget varname;
  Widget varorder;
} e_menu;
typedef struct {
  Widget r_menu;
  Widget clear;
  Widget copy;
  Widget paste;
  Widget transpose;
} r_menu;
typedef struct {
  Widget u_menu;
  Widget sort;
  Widget dep;
  Widget trans;
  Widget trans1;
  Widget trans2;
} u_menu;
typedef struct {
  Widget s_menu;
  Widget list_m;
  Widget mean_m;
  Widget regres_m;
  Widget ttest_m;
  Widget ttpaired_m;
  Widget ttunpaired_m;
  Widget anova1way;
  Widget anova1wayup;
  Widget anova1wayp;
  Widget anova2way;
  Widget anova2waypp;
  Widget anova2waypup;
  Widget anova2wayupup;
  Widget nonpara;
  Widget mannwhitney;
  Widget wilcoxon;
} s_menu;

typedef struct {
  f_menu file;
  e_menu edit;
  r_menu region;
  s_menu stats;
  u_menu utils;
} main_menu;

/* Structure for Toolbar Pushbuttons */
typedef struct {
  Widget open;
  Widget add_col_r;
  Widget add_col_l;
  Widget add_row_top;
  Widget add_row_bot;
  Widget del_row;
  Widget del_col;
} tabline;

/* Structure for marked region */
typedef struct {
  int    row1, col1, row2, col2;
  String *data;
  int    ndata, nrow, ncol;
  short  is_full;
} region;

/* c++ class for sorting */
class sort_class {
  void dep_ok_cb(Widget widget, XtPointer client_data, XtPointer call_data);
public:
  int nsort;            /* Number of sort variables */
  int *sort;            /* points to column numbers of sort variables */
  int ndep;             /* Number of dependent variables */
  int *dep;             /* points to column numbers of dependent variables */
  int ngroup;           /* Number of sorted groups */
  int *group;           /* points to the number of elements in each sort group */

       sort_class();    /* constructor */
       ~sort_class();   /* destructor  */
  void del(int col);    /* delete the col-th. sort variable      */
  void add(int col);    /* adds column col to the sort variables */
  void deldep(int col); /* delete the col-th. dependent variable      */
  void adddep(int col); /* adds column col to the dependent variables */
  void reset_dep();     /* select all possible dependent variables */
};

/* global variables */
XtAppContext app;
main_menu    mymenu;
tabline      tabs;
Widget       StatData, mess_label;
String       colors[] = {"Black", "Blue", "Green", "Red",
			 "Yellow", "Magenta", "Cyan", "White"};
int          last_var_num;   // last variable number used to automatically asign variable names;
sort_class   sort;           // class for sorting of variables;
region       reg;            // structure for marked region;

main(int argc, char *argv[])
{
  Pixmap    bitmap, tip_pixmap[10];
  Widget    toplevel, main_w, form_w, tab_w;
  Widget    menubar, PullDown, PullDown2, PullDown3, PullDown4;
  Widget    mess_a, rowcol_w;
  XmString  label;
  int       i, j, rows, columns;
  String    *rowArrays, **cells;
  short     *colwidth;

  extern void f_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void e_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void r_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void s_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void anova_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void nonpara_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void u_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void trans_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void dragselect_cb(Widget widget, XtPointer client_data, XtPointer call_data);
  extern void entercell_cb(Widget widget, XtPointer client_data, XtPointer call_data);

  void SetPixmap(Widget w, char *pix[]);

  /* Initialization */
  last_var_num=0; // last variable number for automatic var name construction;
  reg.data=new String[1];
  reg.is_full=0;
  reg.ndata=0; reg.nrow=0; reg.ncol=0;

  /* Create Windows and the like */
  toplevel=XtVaAppInitialize(&app, "XmStat", NULL, 0,
			     &argc, argv, NULL, NULL);
  bitmap=XCreatePixmapFromBitmapData(XtDisplay(toplevel),
				     RootWindowOfScreen(XtScreen(toplevel)),
				     stat_icon_bits, stat_icon_width, stat_icon_height,
				     1,0,1);
  XtVaSetValues(toplevel, 
		XmNiconPixmap, bitmap,
		XmNtitle, "xmstat v"VERSION,
		NULL);
  XtAddEventHandler(toplevel, (EventMask)0, True,
		    _XEditResCheckMessages, NULL);

  main_w=XtVaCreateManagedWidget("main", xmMainWindowWidgetClass, toplevel,
				 XmNwidth, DISW,
				 XmNheight, DISH,
			         NULL);
  menubar = XmCreateMenuBar(main_w, "menubar",NULL,0);
  /*   F i l e m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "FilePullDown", NULL,0);
  mymenu.file.f_menu=XtVaCreateManagedWidget("File",
					     xmCascadeButtonWidgetClass, menubar,
					     XmNlabelString, XmStringCreateLocalized("File"),
					     XmNmnemonic, 'F',
					     XmNsubMenuId, PullDown,
					     NULL);
  mymenu.file.n_data=XtVaCreateManagedWidget("New",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'N',
					     NULL);
  mymenu.file.l_data=XtVaCreateManagedWidget("Load",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'L',
					     NULL);
  mymenu.file.s_data=XtVaCreateManagedWidget("Save",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'S',
					     NULL);
  mymenu.file.c_data=XtVaCreateManagedWidget("Close",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'C',
					     NULL);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.file.quit=XtVaCreateManagedWidget("Quit",
					   xmPushButtonGadgetClass, PullDown,
					   XmNmnemonic, 'Q',
					   NULL);
  XtAddCallback(mymenu.file.n_data, XmNactivateCallback, f_cb, (XtPointer)0);
  XtAddCallback(mymenu.file.l_data, XmNactivateCallback, f_cb, (XtPointer)1);
  XtAddCallback(mymenu.file.s_data, XmNactivateCallback, f_cb, (XtPointer)2);
  XtAddCallback(mymenu.file.c_data, XmNactivateCallback, f_cb, (XtPointer)3);
  XtAddCallback(mymenu.file.quit,   XmNactivateCallback, f_cb, (XtPointer)4);
  XtSetSensitive(mymenu.file.n_data, True);
  XtSetSensitive(mymenu.file.l_data, True);
  XtSetSensitive(mymenu.file.s_data, False);
  XtSetSensitive(mymenu.file.c_data, False);
  /*   E d i t m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "EditPullDown", NULL,0);
  mymenu.edit.e_menu=XtVaCreateManagedWidget("Edit",
					     xmCascadeButtonWidgetClass, menubar,
					     XmNlabelString, XmStringCreateLocalized("Edit"),
					     XmNmnemonic, 'E',
					     XmNsubMenuId, PullDown,
					     NULL);
  PullDown2=XmCreatePulldownMenu(PullDown, "addrowPullDown", NULL, 0);
  mymenu.edit.addrow=XtVaCreateManagedWidget("Add Observation",
					     xmCascadeButtonWidgetClass, PullDown,
					     XmNlabelString, XmStringCreateLocalized("Add Observation"),
					     XmNmnemonic, 'O',
					     XmNsubMenuId, PullDown2,
					     NULL);
  mymenu.edit.addrowb=XtVaCreateManagedWidget("Before current",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'B',
					     NULL);
  mymenu.edit.addrowa=XtVaCreateManagedWidget("After current",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'A',
					     NULL);
  mymenu.edit.addrowe=XtVaCreateManagedWidget("at the End",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'E',
					     NULL);
  XtVaSetValues(mymenu.edit.addrowb, XmNuserData, 1, NULL);
  XtVaSetValues(mymenu.edit.addrowa, XmNuserData, 2, NULL);
  XtVaSetValues(mymenu.edit.addrowe, XmNuserData, 3, NULL);
  PullDown2=XmCreatePulldownMenu(PullDown, "addcolPullDown", NULL, 0);
  mymenu.edit.addcol=XtVaCreateManagedWidget("Add Variable",
					     xmCascadeButtonWidgetClass, PullDown,
					     XmNlabelString, XmStringCreateLocalized("Add Variable"),
					     XmNmnemonic, 'V',
					     XmNsubMenuId, PullDown2,
					     NULL);
  mymenu.edit.addcolb=XtVaCreateManagedWidget("Before current",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'B',
					     NULL);
  mymenu.edit.addcola=XtVaCreateManagedWidget("After current",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'A',
					     NULL);
  mymenu.edit.addcole=XtVaCreateManagedWidget("at the End",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'E',
					     NULL);
  XtVaSetValues(mymenu.edit.addcolb, XmNuserData, 1, NULL);
  XtVaSetValues(mymenu.edit.addcola, XmNuserData, 2, NULL);
  XtVaSetValues(mymenu.edit.addcole, XmNuserData, 3, NULL);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.edit.cprow=XtVaCreateManagedWidget("Copy Observation",
					    xmPushButtonGadgetClass, PullDown,
					    NULL);
  mymenu.edit.cpcol=XtVaCreateManagedWidget("Copy Variable",
					    xmPushButtonGadgetClass, PullDown,
					    NULL);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.edit.delrow=XtVaCreateManagedWidget("Delete Observation",
					     xmPushButtonGadgetClass, PullDown,
					     NULL);
  mymenu.edit.delcol=XtVaCreateManagedWidget("Delete Variable",
					     xmPushButtonGadgetClass, PullDown,
					     NULL);
  XtVaCreateManagedWidget("separator", xmSeparatorGadgetClass, PullDown, NULL);
  mymenu.edit.varname=XtVaCreateManagedWidget("Change Variable Name",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'N',
					     NULL);
  mymenu.edit.varorder=XtVaCreateManagedWidget("Change Variable Order",
					     xmPushButtonGadgetClass, PullDown,
					     NULL);
  XtAddCallback(mymenu.edit.addrowb, XmNactivateCallback, e_cb, (XtPointer)0);
  XtAddCallback(mymenu.edit.addrowa, XmNactivateCallback, e_cb, (XtPointer)0);
  XtAddCallback(mymenu.edit.addrowe, XmNactivateCallback, e_cb, (XtPointer)0);
  XtAddCallback(mymenu.edit.addcolb, XmNactivateCallback, e_cb, (XtPointer)1);
  XtAddCallback(mymenu.edit.addcola, XmNactivateCallback, e_cb, (XtPointer)1);
  XtAddCallback(mymenu.edit.addcole, XmNactivateCallback, e_cb, (XtPointer)1);
  XtAddCallback(mymenu.edit.delrow, XmNactivateCallback, e_cb, (XtPointer)2);
  XtAddCallback(mymenu.edit.delcol, XmNactivateCallback, e_cb, (XtPointer)3);
  XtAddCallback(mymenu.edit.varname, XmNactivateCallback, e_cb, (XtPointer)4);
  XtAddCallback(mymenu.edit.varorder, XmNactivateCallback, e_cb, (XtPointer)5);
  XtAddCallback(mymenu.edit.cpcol, XmNactivateCallback, e_cb, (XtPointer)6);
  XtAddCallback(mymenu.edit.cprow, XmNactivateCallback, e_cb, (XtPointer)7);
  XtSetSensitive(mymenu.edit.e_menu, False);
  /*   R e g i o n  m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "RegionPullDown", NULL,0);
  mymenu.region.r_menu=XtVaCreateManagedWidget("Region",
					       xmCascadeButtonWidgetClass, menubar,
					       XmNlabelString, XmStringCreateLocalized("Region"),
					       XmNmnemonic, 'R',
					       XmNsubMenuId, PullDown,
					       NULL);
  mymenu.region.clear=XtVaCreateManagedWidget("cUt",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'U',
					      NULL);
  mymenu.region.copy=XtVaCreateManagedWidget("Copy",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'C',
					      NULL);
  mymenu.region.paste=XtVaCreateManagedWidget("Paste",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'P',
					      NULL);
  mymenu.region.transpose=XtVaCreateManagedWidget("Transpose",
					      xmPushButtonGadgetClass, PullDown,
					      XmNmnemonic, 'T',
					      NULL);
  XtAddCallback(mymenu.region.clear, XmNactivateCallback, r_cb, (XtPointer)0);
  XtAddCallback(mymenu.region.copy, XmNactivateCallback, r_cb, (XtPointer)1);
  XtAddCallback(mymenu.region.paste, XmNactivateCallback, r_cb, (XtPointer)2);
  XtAddCallback(mymenu.region.transpose, XmNactivateCallback, r_cb, (XtPointer)3);
  XtSetSensitive(mymenu.region.r_menu, False);
  XtSetSensitive(mymenu.region.paste, False);

  /*   U t i l s m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "UtilsPullDown", NULL,0);
  mymenu.utils.u_menu=XtVaCreateManagedWidget("Utils",
					     xmCascadeButtonWidgetClass, menubar,
					     XmNlabelString, XmStringCreateLocalized("Utils"),
					     XmNmnemonic, 'U',
					     XmNsubMenuId, PullDown,
					     NULL);
  mymenu.utils.sort=XtVaCreateManagedWidget("Sort Variables",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'S',
					     NULL);
  mymenu.utils.dep=XtVaCreateManagedWidget("Dependent Variables",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'D',
					     NULL);
  PullDown2=XmCreatePulldownMenu(PullDown, "transPullDown", NULL, 0);
  mymenu.utils.trans=XtVaCreateManagedWidget("Transformations",
					     xmCascadeButtonWidgetClass, PullDown,
					     XmNlabelString, XmStringCreateLocalized("Transformations"),
					     XmNmnemonic, 'T',
					     XmNsubMenuId, PullDown2,
					     NULL);
  mymenu.utils.trans1=XtVaCreateManagedWidget("one-variable",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'o',
					     NULL);
  mymenu.utils.trans2=XtVaCreateManagedWidget("two-variables",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 't',
					     NULL);
  XtAddCallback(mymenu.utils.sort,  XmNactivateCallback, u_cb, (XtPointer)0);
  XtAddCallback(mymenu.utils.dep,  XmNactivateCallback, u_cb, (XtPointer)1);
  XtAddCallback(mymenu.utils.trans1,  XmNactivateCallback, trans_cb, (XtPointer)0);
  XtAddCallback(mymenu.utils.trans2,  XmNactivateCallback, trans_cb, (XtPointer)1);
  XtSetSensitive(mymenu.utils.u_menu, False);

  /*   S t a t s m e n u   */;
  PullDown=XmCreatePulldownMenu(menubar, "StatsPullDown", NULL,0);
  mymenu.stats.s_menu=XtVaCreateManagedWidget("Stats",
					     xmCascadeButtonWidgetClass, menubar,
					     XmNlabelString, XmStringCreateLocalized("Statistics"),
					     XmNmnemonic, 'S',
					     XmNsubMenuId, PullDown,
					     NULL);
  mymenu.stats.list_m=XtVaCreateManagedWidget("Data Listing",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'L',
					     NULL);
  mymenu.stats.mean_m=XtVaCreateManagedWidget("Descriptive",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'D',
					     NULL);
  mymenu.stats.regres_m=XtVaCreateManagedWidget("Regression",
					     xmPushButtonGadgetClass, PullDown,
					     XmNmnemonic, 'R',
					     NULL);
  PullDown2=XmCreatePulldownMenu(PullDown, "ttestPullDown", NULL, 0);
  mymenu.stats.ttest_m=XtVaCreateManagedWidget("t-tests",
					       xmCascadeButtonWidgetClass, PullDown,
					       XmNlabelString, XmStringCreateLocalized("t-tests"),
					       XmNmnemonic, 't',
					       XmNsubMenuId, PullDown2,
					       NULL);
  mymenu.stats.ttpaired_m=XtVaCreateManagedWidget("Paired",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'P',
					     NULL);
  mymenu.stats.ttunpaired_m=XtVaCreateManagedWidget("Unpaired",
					     xmPushButtonGadgetClass, PullDown2,
					     XmNmnemonic, 'U',
					     NULL);
  /* One way ANOVA */
  PullDown2=XmCreatePulldownMenu(PullDown, "anova-1-wayPullDown", NULL, 0);
  mymenu.stats.anova1way=XtVaCreateManagedWidget("ANOVA1W",
						 xmCascadeButtonWidgetClass, PullDown,
						 XmNlabelString, XmStringCreateLocalized("ANOVA-one-way"),
						 XmNmnemonic, 'A',
						 XmNsubMenuId, PullDown2,
						 NULL);
  mymenu.stats.anova1wayp=XtVaCreateManagedWidget("Repeated Measures",
						  xmPushButtonGadgetClass, PullDown2,
						  XmNmnemonic, 'R',
						  NULL);
  mymenu.stats.anova1wayup=XtVaCreateManagedWidget("Independent Measures",
						   xmPushButtonGadgetClass, PullDown2,
						   XmNmnemonic, 'I',
						   NULL);
  /* Two way ANOVA */
  PullDown3=XmCreatePulldownMenu(PullDown, "anova-2-wayPullDown", NULL, 0);
  mymenu.stats.anova2way=XtVaCreateManagedWidget("ANOVA2W",
						 xmCascadeButtonWidgetClass, PullDown,
						 XmNlabelString, XmStringCreateLocalized("ANOVA-two-way"),
						 XmNmnemonic, 'V',
						 XmNsubMenuId, PullDown3,
						 NULL);
  mymenu.stats.anova2waypp=XtVaCreateManagedWidget("Repeated-Repeated Measures",
						   xmPushButtonGadgetClass, PullDown3,
						   XmNmnemonic, 'R',
						  NULL);
  mymenu.stats.anova2waypup=XtVaCreateManagedWidget("Independent-Repeated Measures",
						   xmPushButtonGadgetClass, PullDown3,
						   XmNmnemonic, 'M',
						   NULL);
  mymenu.stats.anova2wayupup=XtVaCreateManagedWidget("Independent-Independent Measures",
						   xmPushButtonGadgetClass, PullDown3,
						   XmNmnemonic, 'I',
						   NULL);
  /* Non-Parametric Tests */
  PullDown4=XmCreatePulldownMenu(PullDown, "Non-Para-pulldown", NULL, 0);
  mymenu.stats.nonpara=XtVaCreateManagedWidget("Non-Parametric",
						 xmCascadeButtonWidgetClass, PullDown,
						 XmNlabelString, XmStringCreateLocalized("Non-Parametric"),
						 XmNmnemonic, 'N',
						 XmNsubMenuId, PullDown4,
						 NULL);
  mymenu.stats.wilcoxon=XtVaCreateManagedWidget("Wilcoxon-test",
						xmPushButtonGadgetClass, PullDown4,
						XmNmnemonic, 'W',
						NULL);
  mymenu.stats.mannwhitney=XtVaCreateManagedWidget("Mann-Whitney (U-test)",
						   xmPushButtonGadgetClass, PullDown4,
						   XmNmnemonic, 'U',
						   NULL);

  XtAddCallback(mymenu.stats.list_m,        XmNactivateCallback, s_cb, (XtPointer)0);
  XtAddCallback(mymenu.stats.mean_m,        XmNactivateCallback, s_cb, (XtPointer)1);
  XtAddCallback(mymenu.stats.regres_m,      XmNactivateCallback, s_cb, (XtPointer)2);
  XtAddCallback(mymenu.stats.ttpaired_m,    XmNactivateCallback, s_cb, (XtPointer)3);
  XtAddCallback(mymenu.stats.ttunpaired_m,  XmNactivateCallback, s_cb, (XtPointer)4);
  XtAddCallback(mymenu.stats.anova1wayp,    XmNactivateCallback, anova_cb, (XtPointer)0);
  XtAddCallback(mymenu.stats.anova1wayup,   XmNactivateCallback, anova_cb, (XtPointer)1);
  XtAddCallback(mymenu.stats.anova2waypp,   XmNactivateCallback, anova_cb, (XtPointer)2);
  XtAddCallback(mymenu.stats.anova2waypup,  XmNactivateCallback, anova_cb, (XtPointer)3);
  XtAddCallback(mymenu.stats.anova2wayupup, XmNactivateCallback, anova_cb, (XtPointer)4);
  XtAddCallback(mymenu.stats.wilcoxon,      XmNactivateCallback, nonpara_cb, (XtPointer)0);
  XtAddCallback(mymenu.stats.mannwhitney,   XmNactivateCallback, nonpara_cb, (XtPointer)1);
  XtSetSensitive(mymenu.stats.s_menu, False);
  XtManageChild(menubar);

  /* Form Widget for Toolbar and Matrix */
  form_w = XtVaCreateWidget("Form", xmFormWidgetClass, main_w, NULL);
  tab_w  = XtVaCreateWidget("Toolbar",xmToolbarWidgetClass,form_w,
			    XmNtipPosition, XmTIP_BOTTOM_RIGHT,
			    /* XtVaTypedArg, XmNbackground, XmRString, "#667788", 8, */
			    XmNtopAttachment, XmATTACH_FORM,
			    /* XmNtopOffset, 20, */
			    /* XmNbottomAttachment, XmATTACH_WIDGET, */
			    /* XmNbottomWidget, StatData, */
			    /* XmNbottomOffset, 20, */
			    XmNleftAttachment, XmATTACH_FORM,
			    /* XmNleftOffset, 20, */
			    XmNrightAttachment, XmATTACH_FORM,
			    /* XmNrightOffset, 20, */
			    XtVaTypedArg, XmNtipForeground,XmRString, "black", 6,
			    XtVaTypedArg, XmNtipBackground,XmRString, "yellow", 7,
			    NULL);
  /* tab push button */
  tabs.open = XtVaCreateManagedWidget("PushB", xmPushButtonWidgetClass, tab_w,
			       XtVaTypedArg, XmNtipLabel,XmRString, "Load file", 10,
			       NULL);
  SetPixmap(tabs.open, folder_xpm);
  XtAddCallback(tabs.open, XmNactivateCallback, f_cb, (XtPointer)1);
  
  tabs.add_col_l = XtVaCreateManagedWidget("PushB", xmPushButtonWidgetClass, tab_w,
					   XtVaTypedArg, XmNtipLabel,XmRString, "Add variable before", 20,
					   NULL);
  SetPixmap(tabs.add_col_l, arrleft);
  XtVaSetValues(tabs.add_col_l, XmNuserData, 1, NULL);
  XtAddCallback(tabs.add_col_l, XmNactivateCallback, e_cb, (XtPointer)1);

  tabs.add_col_r = XtVaCreateManagedWidget("PushB", xmPushButtonWidgetClass, tab_w,
					   XtVaTypedArg, XmNtipLabel,XmRString, "Add variable after", 19,
					   NULL);
  SetPixmap(tabs.add_col_r, arrright);
  XtVaSetValues(tabs.add_col_r, XmNuserData, 2, NULL);
  XtAddCallback(tabs.add_col_r, XmNactivateCallback, e_cb, (XtPointer)1);

  tabs.add_row_top = XtVaCreateManagedWidget("PushB", xmPushButtonWidgetClass, tab_w,
					     XtVaTypedArg, XmNtipLabel,XmRString, "Add observation before", 23,
					   NULL);
  SetPixmap(tabs.add_row_top, arrup);
  XtVaSetValues(tabs.add_row_top, XmNuserData, 1, NULL);
  XtAddCallback(tabs.add_row_top, XmNactivateCallback, e_cb, (XtPointer)0);

  tabs.add_row_bot = XtVaCreateManagedWidget("PushB", xmPushButtonWidgetClass, tab_w,
					     XtVaTypedArg, XmNtipLabel,XmRString, "Add observation after", 22,
					     NULL);
  SetPixmap(tabs.add_row_bot, arrdown);
  XtVaSetValues(tabs.add_row_bot, XmNuserData, 2, NULL);
  XtAddCallback(tabs.add_row_bot, XmNactivateCallback, e_cb, (XtPointer)0);

  /* tabs to delete cols or rows */
  tabs.del_row = XtVaCreateManagedWidget("PushB", xmPushButtonWidgetClass, tab_w,
					     XtVaTypedArg, XmNtipLabel,XmRString, "Delete observation", 19,
					     NULL);
  SetPixmap(tabs.del_row, delrow);
  XtAddCallback(tabs.del_row, XmNactivateCallback, e_cb, (XtPointer)2);
  tabs.del_col = XtVaCreateManagedWidget("PushB", xmPushButtonWidgetClass, tab_w,
					     XtVaTypedArg, XmNtipLabel,XmRString, "Delete variable", 16,
					     NULL);
  SetPixmap(tabs.del_col, delcol);
  XtAddCallback(tabs.del_col, XmNactivateCallback, e_cb, (XtPointer)3);

  /* Sensitivity */
  XtSetSensitive(tabs.open, True);
  XtSetSensitive(tabs.add_col_l, False);
  XtSetSensitive(tabs.add_col_r, False);
  XtSetSensitive(tabs.add_row_top, False);
  XtSetSensitive(tabs.add_row_bot, False);
  XtSetSensitive(tabs.del_row, False);
  XtSetSensitive(tabs.del_col, False);
  XtManageChild(tab_w);

  /* Create a Matrix widget. */
  rows=1; columns=1;
  cells = (String **)XtMalloc(rows * sizeof(String *));
  rowArrays = (String *)XtMalloc(rows * columns * sizeof(String));

  for (i = 0; i < rows; i++) {
    cells[i] = &rowArrays[i * columns];
    for (j = 0; j < columns; j++) {
      rowArrays[i * columns + j] = (String) XtNewString(EMPTY);
    }
  }

  colwidth=new short [columns];
  for (i=0; i<columns; i++) colwidth[i]=COLWIDTH;

  StatData=XtVaCreateWidget("mw",
			    xbaeMatrixWidgetClass, form_w,
			    XmNcolumnWidths, colwidth,
			    XmNcells, cells,
			    XmNcolumns, columns,
			    XmNrows, rows,
			    XmNallowColumnResize, True,
			    XmNselectionPolicy, XmEXTENDED_SELECT,
			    XmNtopAttachment, XmATTACH_WIDGET,
			    XmNtopWidget, tab_w,
			    /* XmNtopOffset, 10, */
			    XmNbottomAttachment, XmATTACH_FORM,
			    /* XmNbottomOffset, 20, */
			    XmNleftAttachment, XmATTACH_FORM,
			    /* XmNleftOffset, 20, */
			    XmNrightAttachment, XmATTACH_FORM,
			    /* XmNrightOffset, 20, */
			    NULL);
  XtAddCallback(StatData, XmNselectCellCallback, dragselect_cb, NULL);
  XtAddCallback(StatData, XmNenterCellCallback, entercell_cb, NULL);
  /* XtManageChild(StatData); */ // do not manage the StatData Widget here;
  XtManageChild(form_w);
  
  /* Messagearea */
  mess_a=XtVaCreateManagedWidget("frame", xmFrameWidgetClass, main_w,
				 XmNshadowType, XmSHADOW_OUT,
				 NULL);
  rowcol_w=XtVaCreateManagedWidget("mess_a", xmRowColumnWidgetClass, mess_a,
				   XmNrowColumnType, XmWORK_AREA,
				   XmNorientation, XmHORIZONTAL,
				   NULL);
  mess_label=XtVaCreateManagedWidget("mess_l", xmLabelGadgetClass, rowcol_w,
				     XmNlabelString, XmStringCreateLocalized(" "),
				     NULL);
  XmMainWindowSetAreas(main_w, menubar, NULL, NULL, NULL, form_w);
  XtVaSetValues(main_w, XmNmessageWindow, mess_a, NULL);

  /* Free memory */
  delete [] colwidth;
  XtFree((char *) rowArrays);
  XtFree((char *) cells);

  XtRealizeWidget(toplevel);
  XtAppMainLoop(app);
}


/* .................................. */
/* ... class sort_class functions ... */
/* .................................. */

/* public functions */
/* constructor */
sort_class::sort_class() {
  nsort  = 0;
  sort   = new int [1];
  ndep   = 0;
  dep    = new int [1];
  ngroup = 1;
  group  = new int [1];
}

/* destructor  */
sort_class::~sort_class() {
  nsort  = 0;
  delete [] sort;
  ndep   = 0;
  delete [] dep;
  ngroup = 0;
  delete [] group;
}

/* delete the col-th. sort variable */
void sort_class::del(int col) {
  int i;
  for (i=col; i<nsort-1; i++) sort[i]=sort[i+1];
  nsort--;
  if (nsort>0) sort=(int *)realloc(sort, nsort*sizeof(int));
}

/* adds column col to the sort variables */
void sort_class::add(int col) {
  int   i, is_dep;
  // first, add to sort;
  nsort++;
  sort=(int *)realloc(sort, nsort*sizeof(int));
  sort[nsort-1]=col;
  // then, delete from dep if it was in dep;
  is_dep=-1;
  for (i=0; i<ndep; i++) {
    if (col==dep[i]) is_dep=i;
  }
  if (is_dep>0) {
    for (i=is_dep; i<ndep-1; i++) dep[i]=dep[i+1];
    ndep--;
    if (ndep>0) dep=(int *)realloc(dep, ndep*sizeof(int));
  }
}

/* delete the col-th. dependent variable */
void sort_class::deldep(int col) {
  int i;
  for (i=col; i<ndep-1; i++) dep[i]=dep[i+1];
  ndep--;
  if (ndep>0) dep=(int *)realloc(dep, ndep*sizeof(int));
}

/* adds column col to the dependent variables */
void sort_class::adddep(int col) {
  int   i, is_sort;
  // first, add to dep;
  ndep++;
  dep=(int *)realloc(dep, ndep*sizeof(int));
  dep[ndep-1]=col;
  // then, delete from sort if it was in sort;
  is_sort=-1;
  for (i=0; i<nsort; i++) if (col==sort[i]) is_sort=i;
  if (is_sort>0) {
    for (i=is_sort; i<nsort-1; i++) sort[i]=sort[i+1];
    nsort--;
    if (nsort>0) sort=(int *)realloc(sort, nsort*sizeof(int));
  }
}

/* select all posible dependent variables */
void sort_class::reset_dep() {
  int   i, j, cols;
  short is_sort;

  XtVaGetValues(StatData, XmNcolumns, &cols, NULL);
  
  ndep=0;
  for (i=0; i<cols; i++) {
    is_sort=0;
    for (j=0; j<nsort; j++) if (i==sort[j]) is_sort=1;
    if (!is_sort) {
      /* add to dep */
      ndep++;
      dep=(int *)realloc(dep, ndep*sizeof(int));
      dep[ndep-1]=i;
    }
  }
}

/* SetPixmap procedure */
void SetPixmap(Widget w, char *pix[]) {
  Pixmap bitmap;
  int rc;
  rc=XpmCreatePixmapFromData(XtDisplay(w),
			     RootWindowOfScreen(XtScreen(w)),
			     pix, &bitmap, NULL, NULL);
  if (rc==0)
    XtVaSetValues(w, XmNlabelType, XmPIXMAP,
		  XmNlabelPixmap, bitmap,
		  NULL);
}
